// @flow
/**
* @Author: wansong
* @Date:   2016-06-15T10:34:01+08:00
* @Email:  betterofsong@gmail.com
*/

import { AsyncStorage, NativeModules } from 'react-native';
import URI from 'urijs';
import md5 from 'md5';
import makePromise from 'abortable-promise';
import Logger from 'zhike-mobile-logger';
import ErrorMsg from 'zhike-mobile-error';
import _ from 'lodash';

const _KEY_WHICH_API = '_KEY_WHICH_API';

const Api = function () {
  this.whichApi = null;
  this.NAME = null;
  this.APP_UPDATE_VER = null;
  this.APP_ID = null;

  this.CDNPrefixPROD = '';
  this.CDNPrefixDEV = '';
  this.CDNPrefixSIM = '';

  this.site = '';
  this.hmsr = '';
  this.ua = '';

  this.apiIndex().catch(error => console.log('ignore this: ' ,error));
};

Api.prototype.config = function (appInfo: Object) {
  if (appInfo) {
    const keys = Object.keys(appInfo);
    for (let i = keys.length - 1; i >= 0; i--) {
      const key = keys[i];
      this[key] = appInfo[key];
    }
  }
};

Api.prototype.setUrls = function (
{
  API_ZHIKE_PROD, API_ZHIKE_DEV, API_ZHIKE_SIM,
  API_ZHIKE2_PROD, API_ZHIKE2_DEV, API_ZHIKE2_SIM,
  ACCOUNT_PROD, ACCOUNT_DEV, ACCOUNT_SIM,
  PRACTICE_PROD, PRACTICE_DEV, PRACTICE_SIM,
  OL_PRACTICE_PROD, OL_PRACTICE_DEV, OL_PRACTICE_SIM,
  COMMUNITY_PROD, COMMUNITY_DEV, COMMUNITY_SIM,
  PROMOTION_PROD, PROMOTION_DEV, PROMOTION_SIM,
  CDN_PREF_PROD, CDN_PREF_DEV, CDN_PREF_SIM,
  VIDEO_URL_PREFIX,
}
  : {
  API_ZHIKE_PROD:string, API_ZHIKE_DEV:string, API_ZHIKE_SIM:string,
  API_ZHIKE2_PROD:string, API_ZHIKE2_DEV:string, API_ZHIKE2_SIM:string,
  ACCOUNT_PROD:string, ACCOUNT_DEV:string, ACCOUNT_SIM:string,
  PRACTICE_PROD:string, PRACTICE_DEV:string, PRACTICE_SIM:string,
  OL_PRACTICE_PROD:string, OL_PRACTICE_DEV:string, OL_PRACTICE_SIM:string,
  COMMUNITY_PROD:string, COMMUNITY_DEV:string, COMMUNITY_SIM:string,
  PROMOTION_PROD:string, PROMOTION_DEV:string, PROMOTION_SIM:string,
  CDN_PREF_PROD:string, CDN_PREF_DEV:string, CDN_PREF_SIM:string,
  VIDEO_URL_PREFIX:string,
}) {
  this.CDN_PREF_URLS = [CDN_PREF_PROD, CDN_PREF_SIM, CDN_PREF_DEV];
  this.API_ZHIKE_URLS = [API_ZHIKE_PROD, API_ZHIKE_SIM, API_ZHIKE_DEV];
  this.API_ZHIKE2_URLS = [API_ZHIKE2_PROD, API_ZHIKE2_SIM, API_ZHIKE2_DEV];
  this.ACCOUNT_URLS = [ACCOUNT_PROD, ACCOUNT_SIM, ACCOUNT_DEV];
  this.PRACTICE_URLS = [PRACTICE_PROD, PRACTICE_SIM, PRACTICE_DEV];
  this.OL_PRACTICE_URLS = [OL_PRACTICE_PROD, OL_PRACTICE_SIM, OL_PRACTICE_DEV];
  this.API_COMMUNITY_URLS = [COMMUNITY_PROD, COMMUNITY_SIM, COMMUNITY_DEV];
  this.PROMOTION_URLS = [PROMOTION_PROD, PROMOTION_SIM, PROMOTION_DEV];
  this.VIDEO_URL_PREFIX = VIDEO_URL_PREFIX;

  this.CDNPrefixPROD = CDN_PREF_PROD;
  this.CDNPrefixSIM = CDN_PREF_SIM;
  this.CDNPrefixDEV = CDN_PREF_DEV;
};

export const ApiBase = Api;

Api.replacePort = function (port:number) {
  return url =>
    ex.apiIndex()
    .then(whichApi => (whichApi === 0 || whichApi === 1 ? url : URI(url).port(`${port}`).toString()))
    .catch((error) => {
      console.warn(`failed to get whichApi, error: ${error}`);
    })
  ;
};

Api.trimJson = function (obj:Object) {
  if (!obj ||
    !{}.hasOwnProperty.call(obj, 'code') ||
    !{}.hasOwnProperty.call(obj, 'data')) {
    return obj;
  }
  if (Object.keys(obj).length !== 2) {
    return obj;
  }
  return Api.trimJson(obj.data);
};

const LOG = (
  level:string,
  selector:string,
  url:string,
  requestInfo?:Object|null,
  resp:Object,
  error:Object|null, ...ctx:Array<any>) => {
  console[level](`@${selector} ${level} url: ${url}`, requestInfo, resp, error, ctx);
};

Api.prototype.setApiEnv = function (which:number, callback:(fuck:boolean)=>void) {
  which = Math.max(0, which);
  AsyncStorage.setItem(_KEY_WHICH_API, `${which}`, (error) => {
    if (error) {
      console.error(
        `failed to setItem for key(${_KEY_WHICH_API}), value(${which}), error(${JSON.stringify(error)})`
        );
      callback(false);
    } else {
      this.whichApi = parseInt(which, 10);
      callback(true);
    }
  });
};

Api.prototype.apiIndex = function () {
  return new Promise((resolve, reject) => {
    if (this.whichApi != null) {
      resolve(this.whichApi);
    } else {
      AsyncStorage.getItem(_KEY_WHICH_API).then((ret) => {
        const apiIndex = parseInt(ret || 0, 10);
        this.whichApi = apiIndex;
        resolve(apiIndex);
      }, (error) => {
        reject(error);
      });
    }
  });
};

Api.prototype.currentZhikeApiString = function () {
  return this.apiIndex().then(whichApi =>
     this.API_ZHIKE_URLS[whichApi]
  ).catch((error) => {
    console.error(`failed to get apiIndex, error: ${error}`);
  });
};

Api.prototype.zhikeApiString2 = function () {
  return this.apiIndex().then(whichApi =>
     this.API_ZHIKE2_URLS[whichApi]
  ).catch((error) => {
    console.error(`failed to get apiIndex, error: ${error}`);
  });
};

Api.prototype.apiIndexMemory = function () {
  return this.whichApi || 0;
};

Api.prototype.currentCommunityApiString = function () {
  return this.apiIndex().then(whichApi =>
     this.API_COMMUNITY_URLS[whichApi]
  ).catch((error) => {
    console.error(`failed to get apiIndex, error: ${error}`);
  });
};
Api.prototype.currentOrderList = function () {
  return this.apiIndex().then(whichApi =>
     this.PROMOTION_URLS[whichApi]
  ).catch((error) => {
    console.error(`failed to get apiIndex, error: ${error}`);
  });
};

Api.prototype.currentAccountString = function () {
  return this.apiIndex().then(whichApi =>
     this.ACCOUNT_URLS[whichApi]
  ).catch((error) => {
    console.error(`failed to get apiIndex, error: ${error}`);
  });
};

Api.prototype.currentCDNPrefix = function () {
  return this.apiIndex().then(
    whichApi => this.CDN_PREF_URLS[whichApi]
  ).catch((error) => {
    console.error(`failed to get apiIndex, error: ${error}`);
  });
};

Api.prototype.CDNPrefix = function () {
  return this.CDN_PREF_URLS[this.apiIndexMemory()];
};

Api.prototype.CDNVideoUrlPrefix = function () {
  return this.VIDEO_URL_PREFIX;
};

Api.prototype.currentPracticeUrl = function () {
  return this.apiIndex().then(
    whichApi => this.PRACTICE_URLS[whichApi]
  ).catch((error) => {
    console.error(`failed to get apiIndex, error: ${error}`);
  });
};

Api.prototype.loadUrl = function (
  apiOrAccountOrPractice: bool | () => Promise<string>,
  transformUrl:(input:string) => string,
  method:string,
  path:string,
  params_:Object | () => Object,
  extraArgs?:Object) {

  const { timeout = 20000, cookie } = extraArgs || {};

  let params: Object = {};
  if (typeof params_ === 'object') {
    Object.keys(params_).forEach(
      (k) => {
        if (params_[k] !== undefined) {
          params[k] = params_[k];
        }
      }
    );
  } else {
    params = params_;
  }
  return makePromise(
    this.loadUrlImp(
      apiOrAccountOrPractice,
      transformUrl,
      method,
      path,
      params,
      otherCookie => `${otherCookie}${cookie ? '; ' : ''}${cookie || ''}`
      ),
    timeout,
  );
};

Api.prototype.loadUrlImp = function (apiOrAccountOrPractice, transformUrl, method, path, params, cookie) {
  if (typeof cookie == 'function') {
    return new Promise((resolve, reject) => {
      NativeModules.ZKCookieManager.getCookieAppName(
        this.NAME,
        this.APP_UPDATE_VER,
        this.site,
        this.hmsr,
        false,
        (error, cookieStr) => {
          if (cookieStr) {
            console.log('got cookie: ', cookieStr);
            resolve(cookieStr);
          } else {
            reject(ErrorMsg.ERR_COOKIE_UNACCESSABLE);
          }
        });
    })
    .then(cookie_ => this.loadUrlImp(apiOrAccountOrPractice, transformUrl, method, path, params, cookie(cookie_)))
    .catch((err) => {
      if (err === ErrorMsg.ERR_COOKIE_UNACCESSABLE) {
        console.error('failed to get cookie, error:', err);
      }
      return Promise.reject(err);
    });
  }

  const urlString = (typeof apiOrAccountOrPractice === 'function' && apiOrAccountOrPractice())
  || (apiOrAccountOrPractice ? this.currentApiString() : this.currentAccountString());

  return urlString
  .then(url => (transformUrl ? transformUrl(url) : url))
  .then((api) => {
    let uri:Object;
    const headers:Object = { 'User-Agent': `appversion/10241024; source/app-ios; ${this.ua}` };
    type BodyType = ?Object;
    let body:BodyType;

    const requestInfo:{method:string, headers?:Object, body?:BodyType} = { method };

    headers.Cookie = cookie;

    if (method.toLowerCase() === 'get') {
      const args = typeof params === 'function' ? params() : params;
      uri = args ?
      URI(api).path(URI.joinPaths(api, path)).search(args)
      : URI(api).path(URI.joinPaths(api, path));
    } else {
      uri = URI(api).path(URI.joinPaths(api, path));
      const queryStr = URI(path).query();
      if (queryStr.length) {
        uri.search(queryStr);
      }
      // FIXME: should be given through parameter
      if (typeof params === 'function') {
        headers['Content-Type'] = 'multipart/form-data';
        body = params();
      } else {
        headers['Content-Type'] = 'application/x-www-form-urlencoded';
        // headers['Content-Type'] = 'application/json';
        body = params ? URI('').search(params).query() : undefined;
        // body = JSON.stringify(params);
      }
    }
    if (headers) {
      requestInfo.headers = headers;
    }
    requestInfo.body = body;

    return fetch(uri.toString(), requestInfo)
    .then((resp) => {
      const statusCode = resp.status;
      const statusFmily = Math.floor(statusCode / 100);
      if (statusFmily !== 2) {
        return Promise.reject(resp);
      } else {
        return resp.json(); }
    })
    .then((responseData) => {
      LOG('log', 'loadUrl', uri.toString(), requestInfo, responseData, null, ['verbosed']);
      if (Object.prototype.hasOwnProperty.call(responseData, 'code') && responseData.code === 0) {
        if (process.env.NODE_ENV === 'development') {
          return Api.trimJson(responseData.data);
        } else {
          return responseData.data;
        }
      } else if (Object.prototype.hasOwnProperty.call(responseData, 'status') &&
        Object.prototype.hasOwnProperty.call(responseData.status, 'code') &&
        responseData.status.code === 0) {
        if (process.env.NODE_ENV === 'development') {
          return Api.trimJson(responseData.data);
        } else {
          return responseData.data;
        }
      } else {
        LOG('warn', 'loadUrl', uri.toString(), requestInfo, responseData, {});
        Logger.error('error loading url - code not 0: %s, %s', uri.toString() || 'no link?', JSON.stringify(responseData) || 'no responseData?');
        return Promise.reject(responseData);
      }
    })
    .catch((error) => {
      LOG('error', 'loadUrl', uri.toString(), requestInfo, {}, error);
      Logger.error('error loading url - catch errror: %s, %s', uri.toString() || 'no link?', error && error.message || 'no error message');
      if (error instanceof TypeError && error.message && error.message.match(/Network request failed/)) {
        return Promise.reject(ErrorMsg.ERR_NETWORK_UNAVAILABLE);
      }
      if (typeof this.handleError === 'function') {
        this.handleError(uri.toString(), requestInfo, error);
      }
      return Promise.reject(error);
    });
  });
};

// 登录
type LoginQuery = {
  account:string,
  password:string,
  group:string,
  from:string,
};
Api.prototype.login = function (queries:LoginQuery) {
  const { from='toefl', ...other } = queries || {};
  return this.loadUrl(
    false,
    null,
    'POST',
    '/app/signin',
    { from, ...other }
  );
};

Api.prototype.activate = function (extraInfo?:Object) {
  return new Promise((onResolve, onReject) => {
    NativeModules.ZKDeviceMetaManager.getDeviceId((err, deviceId) => {
      if (err || !deviceId) {
        onReject(err);
      } else {
        onResolve(deviceId);
      }
    });
  })
  .then((deviceId) => {
    if (deviceId) {
      // deviceId = '58CAAB8A-C6DD-45B7-A0F0-4D39D9B186F1';
      const appId = this.APP_ID;
      const dateString = Date.now();
      const digest = md5(appId + deviceId + dateString);
      const activateInfo = { appid:appId, idfa:deviceId, t:dateString, sign:digest };
      if (extraInfo) {
        Object.assign(activateInfo, extraInfo);
      }
      Logger.info(`appId, deviceId, dateString, digest, extraInfo: ${JSON.stringify(activateInfo)}`);
      return this.loadUrl(
        () => this.currentZhikeApiString(),
        Api.replacePort(3116),
        'PUT',
        '/api/mobile/idfa',
        activateInfo
        );
    }
  });

// http://124.207.253.189
};

// 获取验证码
Api.prototype.getCaptcha = function (phoneNumber, type) {
  return this.loadUrl(
    false,
    null,
    'POST',
    '/mobile/captchaPhone',
    { 'phone': phoneNumber, 'type': type }
    );
};

// 校验验证码
Api.prototype.verifyCaptcha = function (phoneNumber, captcha) {
  return this.loadUrl(
    false,
    null,
    'POST',
    '/mobile/captchaPhone/check',
    { 'phone': phoneNumber, 'captcha': captcha }
    );
};

// 注册
Api.prototype.register = function (group, source, phone, password, nickname, detail, otherParams) {
  return this.loadUrl(
    false,
    null,
    'POST',
    '/app/signup',
    {
      'group': group,
      'source': source,
      'phone': phone,
      'password': password,
      'nickname': nickname,
      'detail': detail || '斩托福',
      ...(otherParams || {}),
    }
    );
};

Api.prototype.bindPhone = function (
  params:
  { phone:string, captcha:string, password:string, userToken:string }) {
  return this.loadUrl(false, null, 'POST', '/mobile/bind', params);
};

// 忘记密码
Api.prototype.resetPassword = function (group, captcha, phone, password) {
  return this.loadUrl(
    false,
    null,
    'POST',
    '/mobile/reset',
    { 'group': group, 'captcha': captcha, 'phone': phone, 'password': password }
    );
};

// 修改密码
Api.prototype.changePassword = function (params:{ password:string, newPassword:string, token:string }) {
  return this.loadUrl(
    false,
    null,
    'POST',
    '/set/password',
    params
    );
};


Api.prototype.uploadImage = function (dataProvider, params) {
  if (params.w && params.h) {
    const uriPathQuery = URI('/app/upload');
    uriPathQuery.search({ ...params, x:0, y:0 });
    // let path = `/avatar/upload?w=${params.w}&h=${params.h}`;
    // fixme:base url
    return this.loadUrl(false, null, 'POST', uriPathQuery.toString(), dataProvider);
  } else {
    return new Promise((resolve, reject) => {
      reject(`in valide w and h: ${JSON.stringify(params)}`);
    });
  }
};

Api.prototype.uploadFile = function (dataProvider, params) {
  const uriPathQuery = URI('/api/mobile/files');
  uriPathQuery.search(params);
  return this.loadUrl(
    () => this.zhikeApiString2(),
    null,
    'POST',
    uriPathQuery.toString(),
    dataProvider
  );
};

Api.prototype.updateAvatar = function (token, avatar, group) {
  return this.loadUrl(
    false,
    null,
    'POST',
    '/mobile/avatar',
    { userToken:token, avatar, group }
    );
};

Api.prototype.updateNickname = function (token, nickname, group) {
  return this.loadUrl(
    false,
    null,
    'POST',
    '/mobile/nickname',
    { userToken:token, nickname, group }
    );
};

/**
 * [function description]
 * @param  {[type]} token    [description]
 * @param  {json string} feedback [description]
 * @param  {[type]} referer  [description]
 * @return {promise<number>}          [description]
 */
Api.prototype.postFeedback = function (token, feedback, referrer) {
  return this.loadUrl(
    true,
    Api.replacePort(8005),
    'POST',
    'feedback',
    { token, feedback, referrer }
    );
};

/**
 * [fetchAdvertisements description]
 * @param  {[type]} fields          [返回字段]
 * @param  {[type]} sequence        [排序方式： 1 升序， 0：降序]
 * @param  {[type]} position        [广告位置]
 * @param  {[type]} target          [适用终端]
 * @param  {[type]} version         [版本号]
 * @param  {[type]} examinationId=2 [考试 ID]
 * @return {[type]}                 [description]
 */
Api.prototype.fetchAdvertisements = function (fields, target, position) {
  return this.loadUrl(
    () => this.currentZhikeApiString(),
    Api.replacePort(3100),
    'GET',
    'api/mobile/advertisments',
    { field: fields.join(','), target, position }
    );
};

Api.prototype.fetchShareData  = function (fields, position) {
  if (!fields || !position) {
    console.error('invalid input: ', fields, position);
  }
  return this.loadUrl(
    () => this.currentZhikeApiString(),
    Api.replacePort(3100),
    'GET',
    'api/mobile/advertisments',
    { field: fields.join(','),  position }
    );
};

/*
  获取分享内容
*/
Api.prototype.fetchShareInfomation = function () {
  return this.loadUrl(
    () => this.currentZhikeApiString(),
    Api.replacePort(3100),
    'GET',
    '/api/mobile/advertisments',
    { position: 20 }
  );
};


/*
  上传考试计划
 */
Api.prototype.postExamePlanIntention = function (params:
  { token:string, targetScore?:string, examTime?:number }) {
  return this.loadUrl(
    () => this.currentToeflApiString(),
    Api.replacePort(7003),
    'POST',
    'api/mobile/common/intention',
    params
  );
};

/*
 获取用户学习意向
 */
Api.prototype.fetchLearnIntention = function (userToken) {
  return this.loadUrl(
    () => this.currentCommunityApiString(),
    Api.replacePort(7003),
    'GET',
    '/api/mobile/common/intention',
    { token: userToken }
  );
};


/*
  上传设备号
*/
Api.prototype.uploadRegistrationID = function (userToken, deviceID) {
  return this.loadUrl(
    () => this.currentCommunityApiString(),
    ApiBase.replacePort(8005),
    'PUT',
    '/api/mobile/common/user/status',
    {
      type:'device',
      deviceId: deviceID,
      token: userToken,
      examId: 1,
      timezone: '+08:00'
    }
  );
};

// will be a singleton, if not, also works
const ex = new Api();
export default ex;
