// @flow
import { normalize, schema, denormalize } from 'normalizr';
import ApiBaseObj, { ApiBase } from './ApiBase';

function ApiPractice() {
  const self = this;
  Object.assign(this, {
    olPracticeUrl() {
      return self.currentApi()
      .catch(error => console.error('failed to get api index, default to 0'))
      .then(whichApi => self.OL_PRACTICE_URLS[whichApi || 0]);
    },
    getExames() {
      return self.loadUrl(
        () => self.olPracticeUrl(),
        null,
        'GET',
        '/api/exam/types'
      );
    },
    getTextbooks(params:{ examinationId: number }) {
      return self.loadUrl(
        () => self.olPracticeUrl(),
        null,
        'GET',
        '/api/textbooks',
        params
      );
    },
    getQuestionPackages(params: { examinationId:number, textbookId:number, token:string }) {
      return self.loadUrl(
        () => self.olPracticeUrl(),
        null,
        'GET',
        '/api/question-packages',
        params
      );
    },
    getQuestionPackage(params: { questionPackageId:number }) {
      return self.loadUrl(
        () => self.olPracticeUrl(),
        null,
        'GET',
        `/api/question-package/${params && params.questionPackageId}`,
        params
      );
    },
    getTopics(params:{ subjectId:number }) {
      return self.loadUrl(
        () => self.loadUrl(
          () => self.olPracticeUrl(),
          null,
          'GET',
          '/api/exercise/type',
          params
        )
      );
    },
    getQuestions(params:
      {
        subjectId:number,
        tagId:number,
        questionTypeId:number,
        examinationId:number,
        type:number
        }) {
      return self.loadUrl(
        () => self.olPracticeUrl(),
        null,
        'GET',
        '/api/questions',
        params
      );
    },
    // not used currently
    getPractice(practiceId:number) {
      return self.loadUrl(
        () => self.olPracticeUrl(),
        null,
        'GET',
        '/api/practice',
        { practiceId }
        );
    },
    getPackages(examinationId:number = 1) {
      return self.loadUrl(
        () => self.currentPracticeUrl(),
        null,
        'GET',
        '/m/package',
        { examinationId }
      );
    },
    getPackageDetail(packageId:number) {
      return self.loadUrl(
        () => self.currentPracticeUrl(),
        null,
        'GET',
        `/v2/question-package/${packageId}`
        );
    },
    getQuestionsByIds(ids:[number]) {
      return self.loadUrl(
        () => self.currentPracticeUrl(),
        null,
        'GET',
        '/v2/question',
        { ids:ids.join() }
        )
      .then(data => (ids.length > 1 ? data : [data]));
    },
    getQuestionsByPredicate(pred:Object) {
      return self.loadUrl(
        () => self.currentPracticeUrl(),
        null,
        'GET',
        '/m/questions',
        pred
        );
    },
    getSubjects(examinationId = 1) {
      return self.loadUrl(
        () => self.currentPracticeUrl(),
        null,
        'GET',
        '/m/subjects-tag',
        { examinationId }
        );
    },
    getSubjectCounts(subjects:['hearing' | 'reading' | 'writing' | 'speaking']) {
      return self.loadUrl(
        () => self.currentPracticeUrl(),
        null,
        'GET',
        '/m/subject/count',
        { subjectType: subjects.join(',') }
        );
    },
    getTagCount(subjectId:number, examinationId = 2) {
      return self.loadUrl(
        () => self.currentPracticeUrl(),
        null,
        'GET',
        '/m/subject/tag/count',
        { subjectId, examinationId }
        );
    },
  });
}


ApiPractice.prototype = ApiBaseObj;
const singleton = new ApiPractice();

{
  const question = new schema.Entity('questions');
  const practice = new schema.Entity('practices', { questions: new schema.Array(question) });
  const practiceList = new schema.Array(practice);
  const practiceListBySubject = new schema.Array({ practiceList });
  const questionPackage = new schema.Entity('questionPackages', { subjects:practiceListBySubject } );
  singleton.getQuestionPackage.schema = questionPackage;
}

export default singleton;
